#include "ArraySorts.h"


#define SIZE_CORRECTION 1 // adjustment for distance between two pointers
#define ARRAY_THRESHOLD 4 // size of array to sort with insertion sort

namespace PB_ARRAYSORT
{
//----------------------------------------------------------------------------
//    function:		safeRead(istream& sin, Comparable* d, const char* re)
//    description:	safely reads in a DateTime (d) from sin.
//				Re-prompts and re-enters if input is invalid
//    
//    Input:		Comparable* d from sin
// 
//    Called By:	main
// 
//    Parameters:	istream& sin  -- the input stream
//			Comparable* d -- pointer to the object input
//			const char* prompt  -- prompt, if necessary
// 
//    Returns:          void 
//
//    History Log:
//			2/9/08  PB  completed version 1.0
//			5/7/17  PB updated
// ---------------------------------------------------------------------------
	void safeRead(istream& sin, Comparable* d, const char* prompt)
	{
		const int BUFFERSIZE = 256;
		d->input(sin);
		while(!sin)
		{	// read in number--enter loop if fail
			sin.clear();		// clear fail
			sin.ignore(BUFFERSIZE, '\n');	// read past newline
			cout << prompt;  // re-prompt
			d->input(sin);
		}
		// read past newline once input succeeds
		sin.ignore(BUFFERSIZE, '\n');
	}

//----------------------------------------------------------------------------
//    function:	void printArray(ostream & sout, Comparable **a, int size)
//    description:	can print out an array of DateTime *
// 
//    Output:		Comparable* d sout
//
//    Called By:	main
// 
//    Parameters:	ostream& sout  -- the output stream
//			Comparable** a -- array of pointers to the objects
//			int size  -- number of elements in the array
// 
//    Returns:          void 
//
//    History Log:
//			2/9/08  PB  completed version 1.0
//----------------------------------------------------------------------------
	void printArray(ostream & sout, Comparable **a, int size)
	{
		for(int i = 0; i < size; i++)
		{
			sout << i + 1 << ": ";
			a[i]->print(sout);
			sout << endl;
		}
		sout << endl;
	}

	void quickSort(Comparable** array, int fromIndex, int toIndex)
	{
		int size = toIndex - fromIndex;
		if (size <= 3)
			insertionSort(array, fromIndex, toIndex);
		else
		{
			int mid = partition(array, fromIndex, toIndex);
			quickSort(array, fromIndex, mid);
			quickSort(array, mid, toIndex);
		}
	}

	int partition(Comparable** array, int fromIndex, int toIndex)
	{
		//Comparable* from =  arr // to = arr+to ?
		int arrSize = toIndex - fromIndex;
		int begin, end;
		int pivotPoint = static_cast<int>(arrSize / 2);
		SortFirstMiddleLast(array, fromIndex, pivotPoint, toIndex);
		swap(array[pivotPoint], array[toIndex - 1]);
		begin = fromIndex + 1; //arr[0] was found line 114, start with next one.
		end = toIndex - 2;
		pivotPoint = toIndex - 1;
		while (begin <= end)
		{
			while (*array[begin] < *array[pivotPoint])
				begin++;
			while (*array[pivotPoint] < *array[end])
				end--;
			swap(array[begin], array[end]);
			begin++;
			end--;
		}
		return begin; //begin should == end;
	}

	void SortFirstMiddleLast(Comparable** array, int fromIndex,	int midIndex, int toIndex)
	{
		if (*array[midIndex] < *array[fromIndex])
			swap(array[midIndex], array[fromIndex]);
		if (*array[toIndex] < *array[midIndex])
			swap(array[toIndex], array[midIndex]);
		if (*array[midIndex] < *array[fromIndex])
			swap(array[midIndex], array[fromIndex]);
	}

	void insertionSort(Comparable** array, int fromIndex, int toIndex)
	{

		Comparable *temp = array[fromIndex];
		Comparable *value = array[fromIndex];
		for (int i = fromIndex + 1; i <= toIndex; i++)
		{
			temp = array[i]; //3
			int j = i;
			for (j = i; j > 0; j--)
			{
				value = array[j - 1]; //5 
				if (*temp < *value)   //3<5
					array[j] = array[j - 1];
				else break;
			}
			array[j] = temp;
		}

	}
}

