//----------------------------------------------------------------------------
//	File: CTime.cpp
//
//	class: CTime
//	methods:
//		void CTime::setCurrentTime()
//
//
//----------------------------------------------------------------------------
#include "CTime.h"
namespace PB_DATETIME
{
//----------------------------------------------------------------------------
//    Class:		CTime
//			method:		setCurrentTime()
//
//    description:	sets the time from the system clock
//
//    Called By:	constructor, mutators
//
//    History Log:
//			2/9/08  PB  completed version 1.0
// ---------------------------------------------------------------------------
	void CTime::setCurrentTime()
	{
		time_t rawtime;
		tm *currentTimePtr;
		time(&rawtime);
		currentTimePtr = localtime(&rawtime);

		m_hour = currentTimePtr->tm_hour;
		m_minute = currentTimePtr->tm_min;
		m_second = currentTimePtr->tm_sec;
	}
	
//----------------------------------------------------------------------------
//    Class:		CTime
//			method:		CTime() - Default constructor
//
//    description:	Set current time to CTime
//
//    Called By:	
//
//    History Log:
//			10/5/18  Implement function
// ---------------------------------------------------------------------------
	CTime::CTime()
	{
		setCurrentTime();
	}

//----------------------------------------------------------------------------
//    Class:		CTime
//			method:		CTime(int hour, int minute = 0, int second = 0); Constructor with hour, minute, second
//
//    description:	Set time with hours, minute, second
//
//    Called By:	
//
//    History Log:
//			10/5/18  Implement function
// ---------------------------------------------------------------------------
	CTime::CTime(int hour, int minute, int second)
	{
		setCurrentTime();
		setHour(hour);
		setMinute(minute);
		setSecond(second);
	}


//----------------------------------------------------------------------------
//    Class:		CTime
//			method:		input(istream& sin)
//
//    description:	read input with format hh:mm:ss
//
//    Called By:	
//
//    History Log:
//			10/5/18  Implement function
// ---------------------------------------------------------------------------
	void CTime::input(istream& sin)
	{
		short hour = 0;
		short minute = 0;
		short second = 0;

		if (sin >> hour)
		{
			if (sin.peek() == ':')
			{
				sin.get();
				sin >> minute;
				if (sin.peek() == ':')
				{
					sin.get();
					sin >> second;
				}
			}
		}
		setCurrentTime();
		setHour(hour);
		setMinute(minute);
		setSecond(second);

	}

//----------------------------------------------------------------------------
//    Class:		CTime
//			method:		print(ostream& sout)const
//    description:	Print CTime with format hh:mm:ss
//
//    Called By:	
//
//    History Log:
//			10/5/18  Implement function
// ---------------------------------------------------------------------------
	void CTime::print(ostream& sout)const
	{
		sout << m_hour << ":" << m_minute << ":" << m_second;
	}

//----------------------------------------------------------------------------
//    Class:		CTime
//			method:		operator==(const Comparable &other)const
//    description:	Check equal with other CTime
//
//    Called By:	
//
//    History Log:
//			10/5/18  Implement function
// ---------------------------------------------------------------------------
	bool CTime::operator==(const Comparable &other)const
	{
		bool returnValue = false;
		try
		{
			const CTime &otherTime = dynamic_cast<const CTime&>(other);
			if (m_hour == otherTime.m_hour && m_minute == otherTime.m_minute
				&& m_second == otherTime.m_second)
				returnValue = true;
		}
		catch (bad_cast e)
		{
		}
		return returnValue;
	}

//----------------------------------------------------------------------------
//    Class:		CTime
//			method:		operator<(const Comparable &other)const
//    description:	Compare smaller than with other CTime
//
//    Called By:	
//
//    History Log:
//			10/5/18  Implement function
// ---------------------------------------------------------------------------
	bool CTime::operator<(const Comparable &other)const
	{
		bool returnValue = false;
		const CTime &otherTime = dynamic_cast<const CTime&>(other);
		if (otherTime.m_hour > this->m_hour)
		{
			returnValue = true;
		}
		else if (otherTime.m_hour == this->m_hour && otherTime.m_minute > this->m_minute)
		{
			returnValue = true;
		}
		else if (otherTime.m_hour == this->m_hour && otherTime.m_minute == this->m_minute && otherTime.m_second > this->m_second)
		{
			returnValue = true;
		}
		return returnValue;
	}

//----------------------------------------------------------------------------
//    Class:		CTime
//			method:		setHour(short hour)
//    description:	Set hour
//
//    Called By:	
//
//    History Log:
//			10/5/18  Implement function
	void CTime::setHour(short hour)
	{
		m_hour = (hour > 0 && hour < MAXHOUR - 1) ? hour : m_hour;
	}

//----------------------------------------------------------------------------
//    Class:		CTime
//			method:		setMinute(short minute)
//    description:	Set minute
//
//    Called By:	
//
//    History Log:
//			10/5/18  Implement function
	void CTime::setMinute(short minute)
	{
		m_minute = (minute > 0 && minute < MAXMINSEC - 1) ? minute : m_minute;
	}

//----------------------------------------------------------------------------
//    Class:		CTime
//			method:		setSecond(short second)
//    description:	Set second
//
//    Called By:	
//
//    History Log:
//			10/5/18  Implement function
	void CTime::setSecond(short second)
	{
		m_second = (second > 0 && second < MAXMINSEC - 1) ? second : m_second;
	}
//----------------------------------------------------------------------------
//    Class:		
//			method:		operator<<(ostream& sout, const CTime& time)
//    description:	operator <<
//
//    Called By:	
//
//    History Log:
//			10/5/18  Implement function
	ostream& operator<<(ostream& sout, const CTime& time)
	{
		time.print(sout);
		return sout;
	}


//----------------------------------------------------------------------------
//    Class:		
//			method:		operator >> (istream& sin, CTime& time)
//    description:	operator >>
//
//    Called By:	
//
//    History Log:
//			10/5/18  Implement function
	istream& operator >> (istream& sin, CTime& time)
	{
		time.input(sin);
		return sin;
	}
}
